<?php

namespace Drupal\Tests\copyscape\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\node\Entity\NodeType;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;

/**
 * Tests the Copyscape module.
 *
 * @group copyscape
 */
class CopyscapeAdminUserPagesTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['node', 'text', 'copyscape', 'field', 'user'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * Disable strict config schema.
   *
   * @var bool
   */
  protected $strictConfigSchema = FALSE;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    // Make sure to complete the normal setup steps first.
    parent::setUp();

    // Create and log in an administrative user.
    $this->drupalLogin($this->rootUser);
  }

  /**
   * Tests if user can load the front page among other things.
   */
  public function testLoadSite() {
    // Load the front page.
    $this->drupalGet('<front>');

    // Confirm that the site didn't throw a server error or something else.
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet("user");
  }

  /**
   * Tests if an admin user can log in.
   */
  public function testAdminUser() {
    // Log in an administrative user.
    $this->drupalLogin($this->rootUser);

    $this->drupalGet('admin/config');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
  }

  /**
   * Testing to create a node type with a body textfield.
   */
  public function testCreateContentTypes() {
    // Create the article content type with a datetime and a text textfield.
    $node_type = NodeType::create([
      'type' => 'article',
    ]);
    $node_type->save();

    // Add a text field.
    $field_storage = FieldStorageConfig::create([
      'field_name' => 'field_text_long',
      'entity_type' => 'node',
      'type' => 'text_long',
    ]);
    $field_storage->save();

    $field = FieldConfig::create([
      'field_name' => 'field_text_long',
      'entity_type' => 'node',
      'bundle' => 'article',
      'label' => 'Text Long',
    ]);
    $field->save();

    drupal_flush_all_caches();
  }

  /**
   * Test Copyscape admin page to save some settings.
   */
  public function testAdminSettingsForm() {
    $config = $this->config('copyscape.settings');

    // Testing the content form.
    $this->drupalGet('admin/config/copyscape/content');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
    $session->pageTextContains('Reject content if copy percentage is bigger than (in %)');
    $session->pageTextContains('Ignore search from those sites');
    $session->buttonExists("Save configuration");

    $edit = [
      'reject_content' => '1',
      'reject_value' => '23',
    ];

    $this->submitForm($edit, 'Save configuration');
    $session->statusCodeEquals(200);
    $session->pageTextContains('The configuration options have been saved.');

    // Testing the settings form.
    $this->drupalGet('admin/config/copyscape/api');
    $session->statusCodeEquals(200);
    $session->pageTextContains('API URL');
    $session->pageTextContains('API Username');
    $session->pageTextContains('Bypass Roles');
    $session->buttonExists("Save configuration");

    // Adding dummy data to test the form, I'll use a KernelTest to test
    // the actual API connection.
    $edit = [
      'api_url' => $config->get('api_url'),
      'api_user' => $config->get('api_user'),
      'api_key' => $config->get('api_key'),
      'logs' => 1,
      'show_plag_check_as_warning' => 1,
    ];

    $this->submitForm($edit, 'Save configuration');
    $session->statusCodeEquals(200);
    $session->pageTextContains('The configuration options have been saved.');
  }

  /**
   * Testing for copyrighted content.
   */
  public function testCopyrightedContent() {
    /** @var \Drupal\copyscape\Copyscape\Api $api */
    $api = \Drupal::service('copyscape.api');

    $config = $this->config('copyscape.settings');

    // Lets check for copyrighted text, I am testing some content
    // from https://en.wikipedia.org/wiki/Iron_Age
    $value = "The BBC is the world’s leading public service broadcasterWe’re impartial and independent, 
              and every day we create distinctive, world-class programmes and content which inform, educate and 
              entertain millions of people in the UK and around the world.&nbsp;We do this across:&nbsp;A portfolio of 
              television services, including the UK’s most-watched channel BBC One and our multi award-winning channels 
              for children, as well as national and regional television programmes and services across England, Northern 
              Ireland, Scotland and WalesTen UK-wide radio networks, providing the best live music broadcasting in the UK, 
              as well as speech radio which informs, educates and entertains. We also have two national radio services each 
              in Scotland, Wales and Northern Ireland and local radio stations across England and the Channel Islands, 
              providing an invaluable and unique service to listeners across the UKOur digital services including BBC News, 
              Sport, Weather CBBC and CBeebies, iPlayer and BBC Sounds, BBC Red Button and our vast archive";

    $response = $api->textSearchInternet($value);
    if (FALSE === $response) {
      return;
    }
    else {
      $this->assertTrue($response);
    }
  }

  /**
   * Tests if an admin access to pages again.
   */
  public function testAdminUserAfterChanges() {
    // Log in an administrative user.
    $this->drupalLogin($this->rootUser);

    $this->drupalGet('admin/modules');

    $session = $this->assertSession();
    $session->statusCodeEquals(200);
  }

}
